import mongoose, { Model, Types } from 'mongoose';
import { CreateContactDto } from './dto/create-contact.dto';
import { UpdateContactDto } from './dto/update-contact.dto';
import { Contact } from './schema/contact.schema';

export class ContactRepository<ContactDocument extends Contact> {
    constructor(private readonly contactModel: Model<ContactDocument>) { }

    async createEntity(data: CreateContactDto): Promise<Contact> {
        const createdEntity = new this.contactModel({
            ...data,
            _id: new Types.ObjectId()
        });
        return await createdEntity.save();
    }

    async findOneEntity(id: string): Promise<Contact | null> {
        if (!mongoose.Types.ObjectId.isValid(id)) {
            return null;
        }
        return await this.contactModel.findOne({ _id: id, isDeleted: false });
    }

    async updateEntity(id: string, data: UpdateContactDto): Promise<Contact | null> {
        return await this.contactModel.findByIdAndUpdate(id, data, { new: true });
    }

    async deleteEntity(id: string): Promise<boolean> {
        const data = await this.contactModel.findByIdAndUpdate(
            id,
            { isDeleted: true },
            { new: true }
        );
        if (data) return true;
        return false;
    }

    async findAll(): Promise<Contact[]> {
        return await this.contactModel.find({ isDeleted: false });
    }
    async paginate(page: number, limit: number, query?: any) {
        const skip = (page - 1) * limit;
        const total = await this.contactModel.countDocuments({ ...query, isDeleted: false });
        const pageCount = Math.ceil(total / limit);

        const data = await this.contactModel
            .find({ ...query, isDeleted: false })
            .skip(skip)
            .limit(limit)
            .sort({ createdAt: -1 })

        return {
            data: data,
            pageCount: pageCount,
            total: total
        };
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.contactModel.countDocuments({
                isDeleted: false
            });
        } catch (e) {
            return e;
        }
    }
}