import mongoose, { Model, Types } from 'mongoose';
import { CreatePortfolioDto } from './dto/create-portfolio.dto';
import { UpdatePortfolioDto } from './dto/update-portfolio.dto';
import { Portfolio } from './schema/portfolio.schema';


export class PortfolioRepository<PortfolioDocument extends Portfolio> {
    constructor(private readonly model: Model<PortfolioDocument>) { }

    async createEntity(data: CreatePortfolioDto): Promise<Portfolio> {
        const createdEntity = new this.model({
            ...data,
            _id: new Types.ObjectId()
        });
        return await createdEntity.save();
    }

    async findOneEntity(id: string): Promise<Portfolio | null> {
        if (!mongoose.Types.ObjectId.isValid(id)) {
            return null;
        }
        return await this.model.findOne({ _id: id, isDeleted: false });
    }

    async findOneEntityBySlug(urlSlug: string): Promise<Portfolio | null> {
        try {
            return await this.model.findOne({ urlSlug: urlSlug, isDeleted: false });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, data: UpdatePortfolioDto): Promise<Portfolio | null> {
        const updatedEntity = await this.model.findByIdAndUpdate(id, data, { new: true });
        return await this.findOneEntity(id);
    }

    async paginate(page: number, limit: number, query?: any) {
        const skip = (page - 1) * limit;
        const total = await this.model.countDocuments({ ...query, isDeleted: false });
        const pageCount = Math.ceil(total / limit);

        const data = await this.model
            .find({ ...query, isDeleted: false })
            .skip(skip)
            .limit(limit)
            .sort({ position: 'asc' })

        return {
            data: data,
            pageCount: pageCount,
            total: total
        };
    }

    async deleteEntity(id: string): Promise<boolean> {
        const data = await this.model.findByIdAndUpdate(
            id,
            { isDeleted: true },
            { new: true }
        );
        if (data) return true;
        return false;
    }

    async findAll(): Promise<Portfolio[]> {
        return await this.model.find({ 
            isDeleted: false
        }).sort({ position: "asc" });
    }

    async findOneByFilterQuery(query: any): Promise<Portfolio | null> {
        return await this.model.find({ ...query, isDeleted: false }).lean();
    }

    async findSpecificEntity(fields: string): Promise<Portfolio[]> {
        return await this.model.find({
            isDeleted: false
        }).select(fields).sort({ position: "asc" });
    }

    async updateEntityByID(element: any): Promise<any | null> {
        return await this.model.findByIdAndUpdate(element.id, { position: element.position }, { new: true });
    }

}