import {
    BadRequestException,
    Injectable,
    NotFoundException,
} from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { urlSlug } from 'src/utils/common-helper';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { Services } from '../services/schema/services.schema';
import { ServicesRepository } from '../services/services.repository';
import { Solutions } from '../solutions/schema/solutions.schema';
import { SolutionsRepository } from '../solutions/solutions.repository';
import { CreatePortfolioDto } from './dto/create-portfolio.dto';
import { UpdatePortfolioDto } from './dto/update-portfolio.dto';
import { PortfolioRepository } from './portfolio.repository';
import { Portfolio } from './schema/portfolio.schema';

@Injectable()
export class PortfolioService {
    constructor(
        private readonly doSpaceService: DoSpacesService,
        @InjectModel('portfolios') private readonly portfolioModel: Model<Portfolio>,
        @InjectModel('services') private readonly servicesModel: Model<Services>,
        @InjectModel('solutions') private readonly solutionsModel: Model<Solutions>
    ) { }

    private readonly portfolioRepository = new PortfolioRepository(
        this.portfolioModel,
    );
    private readonly servicesRepository = new ServicesRepository(this.servicesModel);
    private readonly solutionsRepository = new SolutionsRepository(this.solutionsModel);

    async create(
        dto: CreatePortfolioDto,
        files: {
            featuredImage?: UploadedMulterFileI;
            images?: UploadedMulterFileI[];
        },
    ): Promise<Services> {
        dto.urlSlug = urlSlug(dto.title);
        const res = await this.portfolioRepository.findOneByFilterQuery({ urlSlug: dto.urlSlug });
        if (Object.keys(res).length) {
            dto.urlSlug = urlSlug(dto.title + "-v2");
        }
        if (files) {
            if (files.featuredImage) {
                const featureImg: any = await this.doSpaceService.uploadFile(
                    files.featuredImage[0],
                    'Portfolio/',
                );
                dto.featuredImage = featureImg;
            }

            if (files.images) {
                const imagesUrl: string[] = [];
                for (const imgFile of files.images) {
                    const portfolioImageUrl: any = await this.doSpaceService.uploadFile(
                        imgFile,
                        'Portfolio/',
                    );
                    imagesUrl.push(portfolioImageUrl);
                }
                dto.images = imagesUrl;
            }
        }

        const data = await this.portfolioRepository.createEntity(dto);
        if (!data) {
            throw new BadRequestException(Constants.CREATE_FAILED);
        }
        return ResponseUtils.successResponseHandler(
            201,
            'Successfully added to Portfolio.',
            'data',
            data,
        );
    }

    async findAll(pageParam?: string, limitParam?: string): Promise<Portfolio[]> {
        const page = parseInt(pageParam, 10) ? parseInt(pageParam, 10) : 1;
        const limit = parseInt(limitParam, 10) ? parseInt(limitParam, 10) : 10;
        const data = await this.portfolioRepository.paginate(page, limit);
        if (!data) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            data,
        );
    }

    async findAllPortfolioFormData(): Promise<any | Error> {
        const services = await this.servicesRepository.findAll();
        const solutions = await this.solutionsRepository.findSpecificEntity('urlSlug title solutionType');
        const data = {
            services, solutions
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            data,
        );
    }

    async findOne(id: string): Promise<Portfolio | null> {
        const data = await this.portfolioRepository.findOneEntity(id);
        if (!data) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Data fetched successfully',
            'data',
            data,
        );
    }

    async update(
        id: string,
        dto: UpdatePortfolioDto,
        files: {
            featuredImage?: UploadedMulterFileI;
            images?: UploadedMulterFileI[];
        },
    ): Promise<Portfolio | null> {
        const res = await this.portfolioRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }
        const featuredImage = (res as Portfolio)?.featuredImage;
        const images = (res as Portfolio)?.images;
        if (files) {
            if (files.featuredImage) {
                const featureImg: any = await this.doSpaceService.uploadFile(
                    files.featuredImage[0],
                    'Portfolio/',
                );
                dto.featuredImage = featureImg;
            } else {
                dto.featuredImage = featuredImage;
            }

            if (files.images) {
                const imagesUrl: string[] = [];
                for (const imgFile of files.images) {
                    const portfolioImageUrl: any = await this.doSpaceService.uploadFile(
                        imgFile,
                        'Portfolio/',
                    );
                    imagesUrl.push(portfolioImageUrl);
                }
                dto.images = imagesUrl;
            } else {
                dto.images = images;
            }
        } else {
            dto.featuredImage = featuredImage;
            dto.images = images;
        }

        dto.urlSlug = urlSlug(dto.title);

        const resSlug = await this.portfolioRepository.findOneByFilterQuery({ urlSlug: dto.urlSlug, _id: { $ne: id } });
        if (Object.keys(resSlug).length) {
            dto.urlSlug = urlSlug(dto.title + "-v2");
        }
        const data = await this.portfolioRepository.updateEntity(id, dto);
        if (!data) {
            throw new BadRequestException(Constants.UPDATE_FAILED);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Data updated successfully',
            'data',
            data,
        );
    }

    async delete(id: string): Promise<void> {
        const res = await this.portfolioRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(Constants.NOT_FOUND);
        }

        const data = await this.portfolioRepository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(Constants.DELETE_FAILED);
        }
        return ResponseUtils.buildDeletedData(
            data,
            200,
            'Data deleted successfully',
        );
    }

    async updatePortfolioPosition(dto: any): Promise<any | Error> {
        if (dto.dataList) {
            for (const element of dto.dataList) {
                await this.portfolioRepository.updateEntityByID(element);
            }
        }

        const data = await this.portfolioRepository.findAll();

        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }
}
