import mongoose, { Model, Types } from 'mongoose';
import { CreateServicesDto } from './dto/create-services.dto';
import { UpdateServicesDto } from './dto/update-services.dto';
import { Services } from './schema/services.schema';


export class ServicesRepository<ServicesDocument extends Services> {
    constructor(private readonly model: Model<ServicesDocument>) { }

    async createEntity(data: CreateServicesDto): Promise<Services> {
        const createdEntity = new this.model({
            ...data,
            _id: new Types.ObjectId()
        });
        return await createdEntity.save();
    }

    async findOneEntity(id: string): Promise<Services | null> {
        if (!mongoose.Types.ObjectId.isValid(id)) {
            return null;
        }
        return await this.model.findOne({ _id: id, isDeleted: false })
    }

    async findOneEntityBySlug(urlSlug: string): Promise<Services | null> {
        try {
            return await this.model.findOne({ urlSlug: urlSlug, isDeleted: false });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, data: UpdateServicesDto): Promise<Services | null> {
        const updatedEntity = await this.model.findByIdAndUpdate(id, data, { new: true });
        return await this.findOneEntity(id);
    }

    async paginate(page: number, limit: number, query?: any) {
        const skip = (page - 1) * limit;
        const total = await this.model.countDocuments({ ...query, isDeleted: false });
        const pageCount = Math.ceil(total / limit);

        const data = await this.model
            .find({ ...query, isDeleted: false })
            .skip(skip)
            .limit(limit)
            .sort({ createdAt: -1 })

        return {
            data: data,
            pageCount: pageCount,
            total: total
        };
    }

    async deleteEntity(id: string): Promise<boolean> {
        const data = await this.model.findByIdAndUpdate(
            id,
            { isDeleted: true },
            { new: true }
        );
        if (data) return true;
        return false;
    }

    async findAll(): Promise<Services[]> {
        return await this.model.find({ isDeleted: false });
    }

    async findOneByFilterQuery(query: any): Promise<Services | null> {
        return await this.model.find({ ...query, isDeleted: false }).lean();
    }

    async findSpecificEntity(fields: string): Promise<Services[]> {
        return await this.model.find({
            isDeleted: false
        }).select(fields).sort({ cretedAt: -1 })
    }
}
