import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { urlSlug } from 'src/utils/common-helper';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { SolutionsServices } from '../solutions-services/schema/solutions-services.schema';
import { SolutionsServicesRepository } from '../solutions-services/solutions-services.repository';
import { Solutions } from '../solutions/schema/solutions.schema';
import { SolutionsRepository } from '../solutions/solutions.repository';
import { CreateServicesDto } from './dto/create-services.dto';
import { UpdateServicesDto } from './dto/update-services.dto';
import { Services } from './schema/services.schema';
import { ServicesRepository } from './services.repository';



@Injectable()
export class ServicesService {
    constructor(private readonly doSpaceService: DoSpacesService,
        @InjectModel('services') private readonly servicesModel: Model<Services>,
        @InjectModel('solutions') private readonly solutionsModel: Model<Solutions>,
        @InjectModel('solutions-services') private readonly SolutionsServicesModel: Model<SolutionsServices>,
    ) { }

    private readonly solutionsServicesRepository = new SolutionsServicesRepository(this.SolutionsServicesModel);

    private readonly solutionsRepository = new SolutionsRepository(this.solutionsModel);

    private readonly servicesRepository = new ServicesRepository(this.servicesModel);

    async create(dto: CreateServicesDto,
        files: {
            image?: UploadedMulterFileI,
            icon?: UploadedMulterFileI
        }): Promise<Services> {


        dto.urlSlug = urlSlug(dto.title);
        const res = await this.servicesRepository.findOneByFilterQuery({ urlSlug: dto.urlSlug });
        if (Object.keys(res).length) {
            dto.urlSlug = urlSlug(dto.title + "-v2");
        }
        if (files && files.image) {
            const img: any = await this.doSpaceService.uploadFile(files.image[0], "Services/");
            dto.image = img;
        }
        if (files && files.icon) {
            const icon: any = await this.doSpaceService.uploadFile(files.icon[0], "Services/");
            dto.icon = icon;
        }

        const service = await this.servicesRepository.createEntity(dto);
        if (!service) {
            throw new BadRequestException(
                Constants.CREATE_FAILED,
            );
        }

        const solutionList = [];
        for (const solution of dto?.solutions) {
            const foundSolution = await this.solutionsRepository.findOneByFilterQuery({ _id: solution });
            if (foundSolution) {
                const solutionObj = {
                    solution: foundSolution,
                    service: service
                }
                solutionList.push(solutionObj);
            }
        }


        // const serviceUpdate = await this.servicesRepository.updateEntity(service?._id.toString(), dto);
        // if (!serviceUpdate) {
        //     throw new BadRequestException(
        //         Constants.UPDATE_FAILED,
        //     );
        // }

        await this.solutionsServicesRepository.createEntities(solutionList);

        const foundSolution = await this.solutionsServicesRepository.findByFilterQuery({ service: service?._id })
        const solutions = foundSolution?.map((solution) => {
            return {
                _id: solution?.solution?._id,
                name: solution?.solution?.title
            }
        });
        const data = {
            service,
            solutions
        }

        return ResponseUtils.successResponseHandler(201, "Successfully added to Services.", "data", data);
    }

    async findAll(pageParam?: string, limitParam?: string): Promise<Services[]> {
        const page = parseInt(pageParam, 10) ? parseInt(pageParam, 10) : 1;
        const limit = parseInt(limitParam, 10) ? parseInt(limitParam, 10) : 10;
        const data = await this.servicesRepository.paginate(page, limit);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND,
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }


    async findOne(id: string): Promise<Services | null> {
        const service = await this.servicesRepository.findOneEntity(id);
        if (!service) {
            throw new NotFoundException(
                Constants.NOT_FOUND,
            );
        }

        const foundSolution = await this.solutionsServicesRepository.findByFilterQuery({ service: service?._id })
        const solutions = foundSolution?.map((solution) => {
            return solution?.solution?._id
        });
        const data = {
            service,
            solutions
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async update(id: string, dto: UpdateServicesDto,
        files: {
            image?: UploadedMulterFileI,
            icon?: UploadedMulterFileI
        }): Promise<Services | null> {
        const res = await this.servicesRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND,
            );
        }

        const foundImage = (res as Services)?.image;

        dto.urlSlug = urlSlug(dto.title);

        const resSlug = await this.servicesRepository.findOneByFilterQuery({ urlSlug: dto.urlSlug, _id: { $ne: id } });
        if (Object.keys(resSlug).length) {
            dto.urlSlug = urlSlug(dto.title + "-v2");
        }

        if (files && files.image) {
            const img: any = await this.doSpaceService.uploadFile(files.image[0], "Services/");
            dto.image = img;
        } else {
            dto.image = foundImage;
        }

        const foundIcon = (res as Services)?.icon;

        if (files && files.icon) {
            const icon: any = await this.doSpaceService.uploadFile(files.icon[0], "Services/");
            dto.icon = icon;
        } else {
            dto.icon = foundIcon;
        }

        const service = await this.servicesRepository.updateEntity(id, dto);
        if (!service) {
            throw new BadRequestException(
                Constants.UPDATE_FAILED,
            );
        }

        await this.solutionsServicesRepository.bulkDelete(res);

        const solutionList = [];
        for (const solution of dto?.solutions) {
            const foundSolution = await this.solutionsRepository.findOneByFilterQuery({ _id: solution });
            if (foundSolution) {
                const solutionObj = {
                    solution: foundSolution,
                    service: service
                }
                solutionList.push(solutionObj);
            }
        }

        await this.solutionsServicesRepository.createEntities(solutionList);
        const foundSolution = await this.solutionsServicesRepository.findByFilterQuery({ service: service?._id })
        const solutions = foundSolution?.map((solution) => {
            return solution?.solution?._id
        });
        const data = {
            service,
            solutions
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async delete(id: string): Promise<void> {
        const res = await this.servicesRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND,
            );
        }

        const data = await this.servicesRepository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(
                Constants.DELETE_FAILED,
            );
        }

        await this.solutionsServicesRepository.bulkDelete(res);
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}