import { BadRequestException, Injectable, NotFoundException } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { CreateSocialLinksDto } from './dto/create-social-links.dto';
import { UpdateSocialLinksDto } from './dto/update-social-links.dto';
import { SocialLinks } from './schema/social-links.schema';
import { SocialLinksRepository } from './social-links.repository';

@Injectable()
export class SocialLinksService {
    constructor(private readonly doSpaceService: DoSpacesService,
        @InjectModel('social_links') private readonly socialLinksModel: Model<SocialLinks>) { }

    private readonly socialLinksRepository = new SocialLinksRepository(this.socialLinksModel);

    async create(
        dto: CreateSocialLinksDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<SocialLinks> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "SocialLinks/");
            dto.image = image;
        }

        const data = await this.socialLinksRepository.createEntity(dto);
        if (!data) {
            throw new BadRequestException(
                Constants.CREATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(201, "Successfully added to SocialLinks.", "data", data);
    }

    async findAll(): Promise<SocialLinks[]> {
        const data = await this.socialLinksRepository.findAll();
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<SocialLinks | null> {
        const data = await this.socialLinksRepository.findOneEntity(id);
        if (!data) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async update(
        id: string, 
        dto: UpdateSocialLinksDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<SocialLinks | null> {
        const res = await this.socialLinksRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }
        const foundImage = (res as SocialLinks)?.image;
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "SocialLinks/");
            dto.image = image;
        } else {
            dto.image = foundImage;
        }

        const data = await this.socialLinksRepository.updateEntity(id, dto);
        if (!data) {
            throw new BadRequestException(
                Constants.UPDATE_FAILED
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }

    async delete(id: string): Promise<void> {
        const res = await this.socialLinksRepository.findOneEntity(id);
        if (!res) {
            throw new NotFoundException(
                Constants.NOT_FOUND
            );
        }

        const data = await this.socialLinksRepository.deleteEntity(id);
        if (!data) {
            throw new BadRequestException(
                Constants.DELETE_FAILED
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}