import mongoose, { Model, Types } from 'mongoose';
import { CreateSolutionsDto } from './dto/create-solutions.dto';
import { UpdateSolutionsDto } from './dto/update-solutions.dto';
import { Solutions } from './schema/solutions.schema';


export class SolutionsRepository<SolutionsDocument extends Solutions> {
    constructor(private readonly model: Model<SolutionsDocument>) { }

    async createEntity(data: CreateSolutionsDto): Promise<Solutions> {
        const createdEntity = new this.model({
            ...data,
            _id: new Types.ObjectId()
        });
        return await createdEntity.save();
    }

    async findOneEntity(id: string): Promise<Solutions | null> {
        if (!mongoose.Types.ObjectId.isValid(id)) {
            return null;
        }
        return await this.model.findOne({ _id: id, isDeleted: false });
    }

    async findOneEntityBySlug(urlSlug: string): Promise<Solutions | null> {
        try {
            return await this.model.findOne({ urlSlug: urlSlug, isDeleted: false });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, data: UpdateSolutionsDto): Promise<Solutions | null> {
        return await this.model.findByIdAndUpdate(id, data, { new: true });
    }

    async paginate(page: number, limit: number, query?: any) {
        const skip = (page - 1) * limit;
        const total = await this.model.countDocuments({ ...query, isDeleted: false });
        const pageCount = Math.ceil(total / limit);

        const data = await this.model
            .find({ ...query, isDeleted: false })
            .skip(skip)
            .limit(limit)
            .sort({ createdAt: -1 })

        return {
            data: data,
            pageCount: pageCount,
            total: total
        };
    }

    async deleteEntity(id: string): Promise<boolean> {
        const data = await this.model.findByIdAndUpdate(
            id,
            { isDeleted: true },
            { new: true }
        );
        if (data) return true;
        return false;
    }

    async findAll(): Promise<Solutions[]> {
        return await this.model.find({ isDeleted: false });
    }

    async findByFilterQuery(query: any): Promise<Solutions[] | null> {
        return await this.model.find({ ...query, isDeleted: false });
    }

    async findOneByFilterQuery(query: any): Promise<Solutions | null> {
        return await this.model.findOne({ ...query, isDeleted: false }).lean();
    }

    async findSpecificEntity(fields: string): Promise<Solutions[]> {
        return await this.model.find({
            isDeleted: false
        }).select(fields).sort({ cretedAt: -1 })
    }

    async findQuerySpecificEntity(): Promise<Solutions[]> {
        return await this.model.find({ "solutionType": "Development", isDeleted: false }).select('description urlSlug title heading image solutionType');
    }
}